@@ -26,6 +26,7 @@ import copy
 from datetime import datetime
 from distutils.version import LooseVersion
 import functools
+import gzip
 import logging
 import logging.handlers
 import os
@@ -36,8 +37,10 @@ import usagestats
 import yaml
 
 from .utils import iteritems, itervalues, unicode_, stderr, UniqueNames, \
-    escape, CommonEqualityMixin, optional_return_type, hsize, join_root, \
-    copyfile
+    escape, optional_return_type, isodatetime, hsize, join_root, copyfile
+
+
+logger = logging.getLogger(__name__.split('.', 1)[0])
 
 
 FILE_READ = 0x01
@@ -47,7 +50,7 @@ FILE_STAT = 0x08
 FILE_LINK = 0x10
 
 
-class File(CommonEqualityMixin):
+class File(object):
     """A file, used at some point during the experiment.
     """
     comment = None
@@ -60,11 +63,14 @@ class File(CommonEqualityMixin):
         return (isinstance(other, File) and
                 self.path == other.path)
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __hash__(self):
         return hash(self.path)
 
 
-class Package(CommonEqualityMixin):
+class Package(object):
     """A distribution package, containing a set of files.
     """
     def __init__(self, name, version, files=None, packfiles=True, size=None):
@@ -74,6 +80,14 @@ class Package(CommonEqualityMixin):
         self.packfiles = packfiles
         self.size = size
 
+    def __eq__(self, other):
+        return (isinstance(other, Package) and
+                self.name == other.name and
+                self.version == other.version)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def add_file(self, file_):
         self.files.append(file_)
 
@@ -163,13 +177,33 @@ class RPZPack(object):
 
         It is up to the caller to remove that file once done.
         """
-        member = copy.copy(self.tar.getmember('METADATA/config.yml'))
+        self._extract_file(self.tar.getmember('METADATA/config.yml'),
+                           target)
+
+    def _extract_file(self, member, target):
+        member = copy.copy(member)
         member.name = str(target.components[-1])
         self.tar.extract(member,
                          path=str(Path.cwd() / target.parent))
         target.chmod(0o644)
         assert target.is_file()
 
+    def _extract_file_gz(self, member, target):
+        f_in = self.tar.extractfile(member)
+        f_in_gz = gzip.open(f_in)
+        f_out = target.open('wb')
+        try:
+            chunk = f_in_gz.read(4096)
+            while len(chunk) == 4096:
+                f_out.write(chunk)
+                chunk = f_in_gz.read(4096)
+            if chunk:
+                f_out.write(chunk)
+        finally:
+            f_out.close()
+            f_in_gz.close()
+            f_in.close()
+
     @contextlib.contextmanager
     def with_config(self):
         """Context manager that extracts the config to  a temporary file.
@@ -188,19 +222,19 @@ class RPZPack(object):
         target = Path(target)
         if self.version == 1:
             member = self.tar.getmember('METADATA/trace.sqlite3')
+            self._extract_file(member, target)
         elif self.version == 2:
             try:
                 member = self.tar.getmember('METADATA/trace.sqlite3.gz')
             except KeyError:
-                member = self.tar.getmember('METADATA/trace.sqlite3')
+                pass
+            else:
+                self._extract_file_gz(member, target)
+                return
+            member = self.tar.getmember('METADATA/trace.sqlite3')
+            self._extract_file(member, target)
         else:
             assert False
-        member = copy.copy(member)
-        member.name = str(target.components[-1])
-        self.tar.extract(member,
-                         path=str(Path.cwd() / target.parent))
-        target.chmod(0o644)
-        assert target.is_file()
 
     @contextlib.contextmanager
     def with_trace(self):
@@ -343,18 +377,18 @@ def load_iofiles(config, runs):
         if name in files:
             if files[name].path != path:
                 old_name, name = name, uniquenames(name)
-                logging.warning("File name appears multiple times: %s\n"
-                                "Using name %s instead",
-                                old_name, name)
+                logger.warning("File name appears multiple times: %s\n"
+                               "Using name %s instead",
+                               old_name, name)
         else:
             uniquenames.insert(name)
         if path in paths:
             if paths[path] == name:
-                logging.warning("File appears multiple times: %s", name)
+                logger.warning("File appears multiple times: %s", name)
             else:
-                logging.warning("Two files have the same path (but different "
-                                "names): %s, %s\nUsing name %s",
-                                name, paths[path], paths[path])
+                logger.warning("Two files have the same path (but different "
+                               "names): %s, %s\nUsing name %s",
+                               name, paths[path], paths[path])
                 name = paths[path]
             files[name].read_runs.update(readers)
             files[name].write_runs.update(writers)
@@ -397,8 +431,8 @@ def load_config(filename, canonical, File=File, Package=Package):
                                 # Deprecated
                                 'input_files', 'output_files'])
     if unknown_keys:
-        logging.warning("Unrecognized sections in configuration: %s",
-                        ', '.join(unknown_keys))
+        logger.warning("Unrecognized sections in configuration: %s",
+                       ', '.join(unknown_keys))
 
     runs = config.get('runs') or []
     packages = read_packages(config.get('packages'), File, Package)
@@ -489,7 +523,7 @@ version: "{format!s}"
                     else ''),
            version=escape(reprozip_version),
            format='0.8',
-           date=datetime.now().isoformat(),
+           date=isodatetime(),
            what=("# It was generated by the packer and you shouldn't need to "
                  "edit it" if canonical
                  else "# You might want to edit this file before running the "
@@ -593,34 +627,34 @@ def setup_logging(tag, verbosity):
     handler.setFormatter(formatter)
 
     # Set up logger
-    logger = logging.root
-    logger.setLevel(min_level)
-    logger.addHandler(handler)
+    rootlogger = logging.root
+    rootlogger.setLevel(min_level)
+    rootlogger.addHandler(handler)
 
     # File logger
-    dotrpz = Path('~/.reprozip').expand_user()
-    try:
-        if not dotrpz.is_dir():
-            dotrpz.mkdir()
-        filehandler = logging.handlers.RotatingFileHandler(str(dotrpz / 'log'),
-                                                           mode='a',
-                                                           delay=False,
-                                                           maxBytes=400000,
-                                                           backupCount=5)
-    except (IOError, OSError):
-        logging.warning("Couldn't create log file %s", dotrpz / 'log')
-    else:
-        filehandler.setFormatter(formatter)
-        filehandler.setLevel(file_level)
-        logger.addHandler(filehandler)
+    if os.environ.get('REPROZIP_NO_LOGFILE', '').lower() in ('', 'false',
+                                                             '0', 'off'):
+        dotrpz = Path('~/.reprozip').expand_user()
+        try:
+            if not dotrpz.is_dir():
+                dotrpz.mkdir()
+            filehandler = logging.handlers.RotatingFileHandler(
+                str(dotrpz / 'log'), mode='a',
+                delay=False, maxBytes=400000, backupCount=5)
+        except (IOError, OSError):
+            logger.warning("Couldn't create log file %s", dotrpz / 'log')
+        else:
+            filehandler.setFormatter(formatter)
+            filehandler.setLevel(file_level)
+            rootlogger.addHandler(filehandler)
 
-        filehandler.emit(logging.root.makeRecord(
-            __name__.split('.', 1)[0],
-            logging.INFO,
-            "(log start)", 0,
-            "Log opened %s %s",
-            (datetime.now().strftime("%Y-%m-%d"), sys.argv),
-            None))
+            filehandler.emit(logging.root.makeRecord(
+                __name__.split('.', 1)[0],
+                logging.INFO,
+                "(log start)", 0,
+                "Log opened %s %s",
+                (datetime.now().strftime("%Y-%m-%d"), sys.argv),
+                None))
 
 
 _usage_report = None
